import requests
import os
import time
import csv
import json
from urllib.parse import urlparse
from concurrent.futures import ThreadPoolExecutor
import threading
from dataclasses import dataclass
from typing import List, Optional


@dataclass
class DownloadResult:
    url: str
    node_id: str
    success: bool
    error: Optional[str] = None
    file_size: Optional[int] = None


class AdvancedIstmatDownloader:
    def __init__(self, csv_file='istmat_links.csv',
                 output_dir='istmat_html',
                 max_workers=3,
                 delay=1):
        self.csv_file = csv_file
        self.output_dir = output_dir
        self.max_workers = max_workers
        self.delay = delay
        self.results: List[DownloadResult] = []
        self.lock = threading.Lock()

        os.makedirs(self.output_dir, exist_ok=True)

        self.session = requests.Session()
        self.session.headers.update({
            'User-Agent': 'Mozilla/5.0 (compatible; istmat-downloader-bot)',
        })

    def load_links(self) -> List[str]:
        """Загружает ссылки из CSV-файла"""
        urls = []
        with open(self.csv_file, 'r', encoding='utf-8') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                if row and row[0].startswith('http'):
                    urls.append(row[0].strip())
        return urls

    def get_downloaded_ids(self) -> set:
        """Получает множество уже загруженных node ID"""
        return {
            filename.split('_')[1].split('.')[0]
            for filename in os.listdir(self.output_dir)
            if filename.startswith('node_') and filename.endswith('.html')
        }

    def extract_node_id(self, url: str) -> str:
        """Извлекает ID ноды из URL"""
        return url.strip().split('/')[-1]

    def download_single_page(self, url: str) -> DownloadResult:
        """Скачивает одну страницу и возвращает результат"""
        node_id = self.extract_node_id(url)
        filename = f"node_{node_id}.html"
        filepath = os.path.join(self.output_dir, filename)

        # Пропуск, если уже скачано
        if os.path.exists(filepath):
            return DownloadResult(url, node_id, True, "File already exists")

        try:
            time.sleep(self.delay)
            response = self.session.get(url, timeout=30)
            response.raise_for_status()

            with open(filepath, 'w', encoding='utf-8') as f:
                f.write(response.text)

            file_size = len(response.text.encode('utf-8'))
            return DownloadResult(url, node_id, True, None, file_size)

        except Exception as e:
            return DownloadResult(url, node_id, False, str(e))

    def save_progress_json(self):
        """Сохраняет прогресс в JSON"""
        progress_file = os.path.join(self.output_dir, 'download_progress.json')
        data = {
            'total': len(self.results),
            'successful': len([r for r in self.results if r.success]),
            'failed': len([r for r in self.results if not r.success]),
            'results': [r.__dict__ for r in self.results]
        }
        with open(progress_file, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)

    def download_all(self):
        """Основная функция скачивания"""
        all_urls = self.load_links()
        downloaded_ids = self.get_downloaded_ids()

        # Оставляем только те, что не скачаны ранее
        urls_to_download = [
            url for url in all_urls
            if self.extract_node_id(url) not in downloaded_ids
        ]

        total = len(urls_to_download)
        print(f"Всего ссылок в CSV: {len(all_urls)}")
        print(f"Пропущено (уже скачано): {len(downloaded_ids)}")
        print(f"Осталось скачать: {total}")

        with ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            futures = [executor.submit(self.download_single_page, url) for url in urls_to_download]

            for i, future in enumerate(futures, 1):
                result = future.result()
                with self.lock:
                    self.results.append(result)

                if result.success:
                    print(f"[{i}/{total}] ✔ {result.url}")
                else:
                    print(f"[{i}/{total}] ❌ {result.url} - {result.error}")

                if i % 50 == 0 or i == total:
                    self.save_progress_json()

        successful = len([r for r in self.results if r.success])
        failed = len([r for r in self.results if not r.success])
        print(f"Загрузка завершена. Успешно: {successful}, Ошибки: {failed}")
        self.save_progress_json()


if __name__ == "__main__":
    downloader = AdvancedIstmatDownloader(
        csv_file='istmat_links.csv',
        output_dir='istmat_html',
        max_workers=2,   # Число потоков
        delay=1.0        # Задержка между запросами в сек
    )
    downloader.download_all()
